#!/usr/bin/env python3

import hashlib


class RpqLua:
    def __init__(self, redisConnection):
        '''Sets Redis connection, load LUA from path'''

        # Set Redis connection
        self.setRedisConnection(redisConnection)

        # Load LUA source
        self.loadSource(self.getLuaPath())

        # Register LUA script
        self.register()

    def getLuaPath(self):
        """
            Returns the LUA path in the filesystem
        """

        import rpq_src
        import pkg_resources

        return pkg_resources.resource_filename('rpq_src', 'redis-priority-queue.lua')

    def file_get_contents(self, filename):
        """
            Get a file content
        """

        with open(filename) as f:
            return f.read()

    def setRedisConnection(self, connection):
        """
            Set a Redis connection
        """

        self.connection = connection

        return True

    def loadSource(self, path):
        """
            Load LUA script source code from a file
        """

        self.source = self.file_get_contents(path)

        return True

    def getSha1(self):
        """
            Calculate sha1 of the LUA source
        """

        d = hashlib.sha1(self.source.encode('utf-8'))
        d.digest()
        return d.hexdigest()

    def exists(self):
        """
            Returns `True` if the LUA script is already loaded into Redis
        """

        # Check if the script exists
        t = self.connection.script_exists(self.sha1)

        if t and t[0]:
            return True

        return False

    def load(self):
        """
            Load the script into Redis
        """

        self.connection.script_load(self.source)

        return True

    def register(self):
        """
            Registers the script
        """

        # Set LUA sha1
        self.sha1 = self.getSha1()

        # Load if needed
        if not self.exists():
            self.load()

        return True

    def eval(self, *args):
        """
            Call the LUA script with the desired arguments and returns the output
        """

        return self.connection.evalsha(self.sha1, 0, *args)



#!/usr/bin/env python3

# redis-priority-queue -> queue monitor
# Author: Gabriel Bordeaux (gabfl)
# Github: https://github.com/gabfl/redis-priority-queue
# Compatible with python 2.7 & 3

import redis
import argparse
import json
from prettytable import PrettyTable

# Global vars
t = None
r = None


def setSortGroups(sortGroups=None):
    """
        Return the sorting groups, either user defined or from the default list
    """

    if sortGroups is None:  # Default groups
        return [('-inf', '+inf'), ('-inf', 100), (101, '+inf')]
    else:
        sortGroups.insert(0, ('-inf', '+inf'))
        return sortGroups


def getCount(queueName, min='-inf', max='+inf'):
    """
        Fetches set count from Redis
    """

    global r

    return r.zcount(queueName, min, max)


def getColumnTitle(min='-inf', max='+inf'):
    """
        Human readable column titles
    """

    if str(min) == '-inf' and str(max) == '+inf':
        return 'Total'
    elif str(min) == '-inf':
        return 'Up to ' + '{0:,}'.format(max)
    elif str(max) == '+inf':
        return 'From ' + '{0:,}'.format(min)
    elif min == max:
        return '{0:,}'.format(min)
    else:
        return '{0:,}'.format(min) + ' to ' + '{0:,}'.format(max)


def setColumnAlign(titles):
    """
        Set PrettyTable column alignment
    """

    global t

    for i, title in enumerate(titles):
        if i == 0:  # First column (title)
            t.align[title] = 'l'  # Left
        else:  # Any other column
            t.align[title] = 'r'  # Right


def monitor(host, port, dbnum, password, sort_groups=None, out='table'):
    global t, r

    # Redis connection
    try:
        r = redis.StrictRedis(host=host, port=port,
                              db=dbnum, password=password)
        r.ping()
    except redis.exceptions.ConnectionError as e:
        import sys

        print('Redis error: %s' % (e))
        sys.exit()

    # Sort groups
    sortGroups = setSortGroups(sort_groups)

    # Column titles (queue name, then a column per sorting group)
    titles = ['Queue name']
    titles.extend([getColumnTitle(sortGroup[0], sortGroup[1])
                   for sortGroup in sortGroups])

    # Create table
    if out == 'table':
        t = PrettyTable(titles)
        setColumnAlign(titles)

    # Get queues
    queueNames = r.smembers('rpq|names')

    # Add a row par queue
    rows = []
    for queueName in sorted(queueNames):
        # Get row
        row = [queueName.decode("utf-8")]
        row.extend([str(getCount(queueName, sortGroup[0], sortGroup[1]))
                    for sortGroup in sortGroups])

        # Add row
        rows.append(row)

    # Print table
    if out == 'table':
        for row in rows:
            # Add thousand separators to all items except the first one
            row = ['{0:,}'.format(int(i)) if i != row[0] else i for i in row]

            t.add_row(row)
        print(t)
    else:
        for row in rows:
            print(','.join(row))


def main():
    # Parse arguments
    parser = argparse.ArgumentParser()
    parser.add_argument("-H", "--host", default="127.0.0.1",
                        help="Redis server host")
    parser.add_argument("-p", "--port", type=int, default=6379,
                        help="Redis server port")
    parser.add_argument("-a", "--auth",
                        help="Redis server authentification")
    parser.add_argument("-n", "--dbnum", type=int, default=0,
                        help="Redis server database number")
    parser.add_argument("-s", "--sort_groups",
                        help="Sort groups", type=json.loads)
    parser.add_argument("-o", "--out",
                        help="Output type", choices=['table', 'csv'],
                        default='table')
    args = parser.parse_args()

    monitor(host=args.host,
            port=args.port,
            dbnum=args.dbnum,
            password=args.auth,
            sort_groups=args.sort_groups,
            out=args.out)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

from .RpqLua import RpqLua


class RpqQueue:

    def __init__(self, redis, queueName):
        """
            Registers RpqLua queue from a Redis connection
        """

        # RpqLua instance
        self.queue = RpqLua(redis)

        # Set queue name
        self.setqueueName(queueName)

    def eval(self, args):
        return self.queue.eval(*args)

    def bytesDecode(self, item):
        """
            Decode Bytes to utf-8 for Python 3 compatibility
        """

        return item.decode("utf-8")

    def setqueueName(self, queueName):
        """
            Define loaded queue name at class level
        """

        self.queueName = queueName

    def push(self, item, priority=None):
        """
            Push an item
        """

        # Build arg list
        args = ['push', self.queueName, item]
        if priority is not None:
            args.append(priority)

        return self.eval(args)

    def pop(self, orderBy='desc'):
        """
            alias for popOne()
        """

        return self.popOne(orderBy)

    def popOne(self, orderBy='desc'):
        """
            Pop an item
        """

        item = self.popMany(orderBy, 1)

        if item:  # There is an item
            return item[0]
        else:
            return None

    def popMany(self, orderBy='desc', numberOfItems=1):
        """
            Pop many items
        """

        items = self.eval(['pop', self.queueName, orderBy, numberOfItems])
        if items:
            # Decode all items from bytes to utf-8
            items = tuple(map(self.bytesDecode, items))

            return items
        else:
            return None

    def peek(self, orderBy='desc'):
        """
            alias for peekOne()
        """

        return self.peekOne(orderBy)

    def peekOne(self, orderBy='desc'):
        """
            Peek an item
        """

        item = self.peekMany(orderBy, 1)

        if item:  # There is an item
            return item[0]
        else:
            return None

    def peekMany(self, orderBy='desc', numberOfItems=1):
        """
            Peek many items
        """

        items = self.eval(['peek', self.queueName, orderBy, numberOfItems])
        if items:
            # Decode all items from bytes to utf-8
            items = tuple(map(self.bytesDecode, items))

            return items
        else:
            return None

    def count(self, priorityMin=None, priorityMax=None):
        """
            Get queue size
        """

        # Build arg list
        args = ['size', self.queueName]
        if priorityMin is not None:
            args.append(priorityMin)
            if priorityMax is not None:
                args.append(priorityMax)

        return self.eval(args)



from setuptools import setup

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except(IOError, ImportError):
    long_description = open('README.md').read()

setup(
    name='rpq',
    version='2.2',
    description='Simple Redis work queue with added features (priorities, pop multiple items at once)',
    long_description=long_description,
    author='Gabriel Bordeaux',
    author_email='pypi@gab.lc',
    url='https://github.com/gabfl/redis-priority-queue',
    license='MIT',
    packages=['rpq', 'rpq_src'],
    package_dir={'rpq': 'clients/python/lib', 'rpq_src': 'src'},
    package_data={
        'rpq_src': ['*.lua'],
    },
    install_requires=['redis', 'argparse',
                      'prettytable'],  # external dependencies
    entry_points={
        'console_scripts': [
            'rpq_monitor = rpq_src.queue_monitor:main',
        ],
    },
    classifiers=[  # see https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Topic :: Software Development',
        'Topic :: Database',
        'Topic :: Database :: Database Engines/Servers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS',
        'Operating System :: POSIX :: Linux',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python',
        #  'Development Status :: 4 - Beta',
        'Development Status :: 5 - Production/Stable',
    ],
)
